from typing import Any, Optional

from pydantic import BaseModel, Field, HttpUrl

from package_schemes.schemes.base import TomlBaseModel


class PyProjectScheme(TomlBaseModel):
    class Project(BaseModel):
        class ProjectLicense(BaseModel):
            file: Optional[str] = None
            text: Optional[str] = None

        class ProjectUrls(BaseModel):
            documentation: Optional[HttpUrl] = None
            repository: Optional[HttpUrl] = None

        name: str
        version: Optional[str] = None
        description: Optional[str] = None
        authors: Optional[list[dict[str, str]]] = Field(default_factory=list)
        maintainers: Optional[list[dict[str, str]]] = Field(default_factory=list)
        license: Optional[ProjectLicense] = None
        keywords: list[str] = Field(default_factory=list)
        classifiers: list[str] = Field(default_factory=list)
        dependencies: list[str] = Field(default_factory=list)
        urls: Optional[ProjectUrls] = None
        optional_dependencies: dict[str, list[str]] = Field(
            alias='optional-dependencies',
            default_factory=dict,
        )
        dependency_groups: dict[str, list[str]] = Field(
            alias='dependency-groups',
            default_factory=dict,
        )

    project: Project



from .packages import Package, PackageMeta
from .poetry_lock import PoetryLock
from .poetry_pyproject import PoetryPyproject
from .projects import Project
from .pyproject import Pyproject
from .uv_lock import UvLockV1

__all__ = (
    'Package',
    'PackageMeta',
    'PoetryLock',
    'PoetryPyproject',
    'Project',
    'UvLockV1',
    'Pyproject',
)



from typing import Any

import toml
from pydantic import BaseModel


class TomlBaseModel(BaseModel):
    @classmethod
    def model_validate_toml(cls, toml_data: str, *args: Any, **kwargs: Any):
        data = toml.loads(toml_data)
        return cls.model_validate(data, *args, **kwargs)



import abc
from pathlib import Path
from typing import Optional

from package_schemes.interfaces.packages import PackageMeta


class BasePyproject(abc.ABC):
    file: Path

    def __init__(self, file: Path) -> None:
        self.file = file

    @abc.abstractmethod
    def get_root_packages_names(self) -> list[str]:
        pass

    @abc.abstractmethod
    def get_package_meta(self, name: str) -> Optional[PackageMeta]:
        pass


class BaseLock(abc.ABC):
    file: Path

    def __init__(self, file: Path) -> None:
        self.file = file

    @abc.abstractmethod
    def get_package_meta(self, name: str) -> Optional[PackageMeta]:
        pass



from typing import Any, Optional

from pydantic import BaseModel, Field, HttpUrl

from package_schemes.schemes.base import TomlBaseModel


class UvLockV1Scheme(TomlBaseModel):
    class ResolutionMarker(BaseModel):
        marker: str

    class Package(BaseModel):
        class Source(BaseModel):
            registry: Optional[HttpUrl] = None
            editable: Optional[str] = None

        class Wheel(BaseModel):
            url: HttpUrl
            hash: str
            size: int

        class Sdist(BaseModel):
            url: HttpUrl
            hash: str
            size: int

        class Dependency(BaseModel):
            name: str
            specifier: Optional[str] = None

        name: str
        version: str
        source: Optional[Source]
        sdist: Optional[Sdist] = None
        wheels: list[Wheel] = Field(default_factory=list)
        dependencies: list[Dependency] = Field(default_factory=list)
        metadata_requires_dist: list[Dependency] = Field(default_factory=list)
        metadata_requires_dev: list[Dependency] = Field(default_factory=list)

        optional_dependencies: dict[str, list[dict[str, Any]]] = Field(
            alias='optional-dependencies',
            default_factory=dict,
        )
        dependency_groups: dict[str, list[dict[str, Any]]] = Field(
            alias='dependency-groups',
            default_factory=dict,
        )

    version: int
    requires_python: Optional[str] = None
    resolution_markers: list[ResolutionMarker] = Field(default_factory=list)
    packages: list[Package] = Field(alias='package', default_factory=list)
